@@ -7,10 +7,6 @@ use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
 use xml::{EmitterConfig, EventReader, EventWriter, ParserConfig};
 
-#[cfg(not(feature = "glib"))]
-use crate::no_glib::{ToVariant, Variant};
-#[cfg(feature = "glib")]
-use glib::{ToVariant, Variant};
 use walkdir::WalkDir;
 
 const FLAG_COMPRESSED: u32 = 1 << 0;
@@ -23,6 +19,9 @@ struct FileData<'a> {
     key: String,
     data: Cow<'a, [u8]>,
     flags: u32,
+
+    /// uncompressed data is zero-terminated
+    /// compressed data is not
     size: u32,
 }
 
@@ -153,20 +152,19 @@ impl<'a> FileData<'a> {
     pub fn key(&self) -> &str {
         &self.key
     }
+}
 
-    pub fn data(&self) -> &Cow<'_, [u8]> {
-        &self.data
-    }
-
-    /// uncompressed data is zero-terminated
-    /// compressed data is not
-    pub fn size(&self) -> u32 {
-        self.size
-    }
-
-    pub fn flags(&self) -> u32 {
-        self.flags
-    }
+/// GResource data value
+///
+/// This is the format in which all GResource files are stored in the GVDB file.
+///
+/// The size is the *uncompressed* size and can be used for verification purposes.
+/// The flags only indicate whether a file is compressed or not. (Compressed = 1)
+#[derive(zvariant::Type, zvariant::Value, zvariant::OwnedValue)]
+pub struct GResourceData {
+    size: u32,
+    flags: u32,
+    data: Vec<u8>,
 }
 
 /// Create a GResource binary file
@@ -380,15 +378,14 @@ impl<'a> GResourceBuilder<'a> {
         let builder = GvdbFileWriter::new();
         let mut table_builder = GvdbHashTableBuilder::new();
 
-        for file_data in self.files {
-            let tuple = vec![
-                file_data.size().to_variant(),
-                file_data.flags().to_variant(),
-                file_data.data().to_variant(),
-            ];
-            let variant = Variant::tuple_from_iter(tuple);
+        for file_data in self.files.into_iter() {
+            let data = GResourceData {
+                size: file_data.size,
+                flags: file_data.flags,
+                data: file_data.data.to_vec(),
+            };
 
-            table_builder.insert_variant(file_data.key(), variant)?;
+            table_builder.insert_value(file_data.key(), zvariant::Value::from(data))?;
         }
 
         Ok(builder.write_to_vec_with_table(table_builder)?)
@@ -510,10 +507,12 @@ mod test {
         let svg2 = table
             .get_value("/gvdb/rs/test/icons/scalable/actions/send-symbolic.svg")
             .unwrap()
-            .child_value(0);
-        let svg2_size = svg2.child_value(0).get::<u32>().unwrap();
-        let svg2_flags = svg2.child_value(1).get::<u32>().unwrap();
-        let svg2_data: &[u8] = &svg2.child_value(2).data_as_bytes();
+            .downcast::<zvariant::Structure>()
+            .unwrap()
+            .into_fields();
+        let svg2_size = *svg2[0].downcast_ref::<u32>().unwrap();
+        let svg2_flags = *svg2[1].downcast_ref::<u32>().unwrap();
+        let svg2_data: &[u8] = &svg2[2].clone().downcast::<Vec<u8>>().unwrap();
 
         assert_eq!(svg2_size, 339);
         assert_eq!(svg2_flags, 0);
