@@ -16,13 +16,20 @@
 //!     let file = GvdbFile::from_file(&path).unwrap();
 //!     let table = file.hash_table().unwrap();
 //!
-//!     let svg1 = table
+//!     #[derive(serde::Deserialize, zvariant::Type)]
+//!     struct SvgData {
+//!         size: u32,
+//!         flags: u32,
+//!         content: Vec<u8>
+//!     }
+//!    
+//!     let value = table
 //!         .get_value("/gvdb/rs/test/online-symbolic.svg")
-//!         .unwrap()
-//!         .child_value(0);
-//!     let svg1_size = svg1.child_value(0).get::<u32>().unwrap();
-//!     let svg1_flags = svg1.child_value(1).get::<u32>().unwrap();
-//!     let svg1_content = svg1.child_value(2).data_as_bytes();
+//!         .unwrap();
+//!     let svg = value.downcast_ref::<zvariant::Structure>().unwrap().fields();
+//!     let svg1_size = svg[0].downcast_ref::<u32>().unwrap();
+//!     let svg1_flags = svg[1].downcast_ref::<u32>().unwrap();
+//!     let svg1_content = svg[2].clone().downcast::<Vec<u8>>().unwrap();
 //!     let svg1_str = std::str::from_utf8(&svg1_content[0..svg1_content.len() - 1]).unwrap();
 //!
 //!     println!("{}", svg1_str);
@@ -32,10 +39,6 @@
 //! Create a simple GVDB file with [`GvdbFileWriter`](crate::write::GvdbFileWriter)
 //!
 //! ```
-//! #[cfg(feature = "glib")]
-//! # use glib::ToVariant;
-//! # #[cfg(not(feature = "glib"))]
-//! # use gvdb::no_glib::ToVariant;
 //! use gvdb::write::{GvdbFileWriter, GvdbHashTableBuilder};
 //!
 //! fn create_gvdb_file() {
@@ -47,7 +50,7 @@
 //!
 //!     let mut table_builder_2 = GvdbHashTableBuilder::new();
 //!     table_builder_2
-//!         .insert_variant("int", 42u32.to_variant())
+//!         .insert("int", 42u32)
 //!         .unwrap();
 //!
 //!     table_builder
@@ -59,22 +62,13 @@
 //!
 //! ## Features
 //!
-//! ### Default
-//!
-//! By default, the `glib` is enabled. You can opt out of this feature by specifying
-//! `default-features = false in the gvdb dependency declaration.
+//! By default, no features are enabled.
 //!
 //! ### `glib`
 //!
-//! By default this crate uses the [glib](https://crates.io/crates/glib) crate to allow reading and
-//! writing `GVariant` data to the gvdb files. By disabling this feature an internal `GVariant`
-//! implementation is used.
-//!
-//! The internal implementation implements the most common serialisation and deserialization
-//! features. For compatibility with glib types it is recommended to enable this feature. Disabling
-//! this feature reduces the number of dependencies and compile time.
-//!
-//! You can find the custom `GVariant` implementation in the [`no_glib`] module.
+//! By default this crate uses the [glib](https://crates.io/crates/zvariant) crate to allow reading
+//! and writing `GVariant` data to the gvdb files. By enabling this feature you can pass GVariants
+//! directly from the glib crate as well.
 //!
 //! ### `gresource`
 //!
@@ -86,6 +80,7 @@
 //! GResource file creation.
 
 #![warn(missing_docs)]
+#![doc = include_str!("../README.md")]
 
 extern crate core;
 
@@ -107,11 +102,6 @@ pub mod read;
 /// See the documentation of [`GvdbFileWriter`](crate::write::GvdbFileWriter) to get started
 pub mod write;
 
-/// `GVariant` implementation that does not depend on glib. This is only available when the `glib`
-/// feature is disabled or the test suite is running.
-#[cfg(any(not(feature = "glib"), test, doc))]
-pub mod no_glib;
-
 #[cfg(test)]
 pub(crate) mod test;
 
