@@ -8,6 +8,9 @@ pub enum GvdbWriterError {
 
     /// An internal inconsistency was found
     Consistency(String),
+
+    /// An error occured when serializing variant data with zvariant
+    ZVariant(zvariant::Error),
 }
 
 impl From<std::io::Error> for GvdbWriterError {
@@ -16,6 +19,12 @@ impl From<std::io::Error> for GvdbWriterError {
     }
 }
 
+impl From<zvariant::Error> for GvdbWriterError {
+    fn from(err: zvariant::Error) -> Self {
+        Self::ZVariant(err)
+    }
+}
+
 impl Display for GvdbWriterError {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -29,6 +38,9 @@ impl Display for GvdbWriterError {
             GvdbWriterError::Consistency(context) => {
                 write!(f, "Internal inconsistency: {}", context)
             }
+            GvdbWriterError::ZVariant(err) => {
+                write!(f, "Error writing ZVariant data: {}", err)
+            }
         }
     }
 }
